<?php

namespace App\Repository\Eloquent;

use App\Http\Requests\GetOTPRequest;
use App\Http\Requests\LoginRequest;
use App\Http\Requests\RegisterRequest;
use App\Http\Requests\User\ChangePassword;
use App\Http\Requests\User\UpdateUser;
use App\Http\Requests\ValidateOTPRequest;
use App\Http\Resources\CompactItemResource;
use App\Http\Services\UserServices;
use App\Models\Item;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Hash;

class UserRepository extends BaseRepository{

    protected $model, $userServices;

    public function __construct(User $model)
    {
        $this->model = $model;
        $this->userServices = new UserServices();
    }


    //---------------------------------------------------------------------------------------------------
    

    public function register(RegisterRequest $request)
    {
        return $this->sendResponse($this->userServices->saveUser($request), 'شما با موفقیت ثبت نام شدید');
    }

    //---------------------------------------------------------------------------------------------------


    public function login(LoginRequest $request)
    {

        if($request->exists('email')){

            if(empty($data = $this->userServices->checkLoginInfo($request, 'email')))
                return $this->sendError('ایمل وارد شده یا رمز عبور اشتباه است');
            else
                return $this->sendResponse($data, 'وارد شدید');
            
        } elseif ($request->exists('phone_number')){

            if(empty($data = $this->userServices->checkLoginInfo($request, 'phone_number')))
                return $this->sendError('شماره وارد شده یا رمز عبور اشتباه است');
            else
                return $this->sendResponse($data, 'وارد شدید');
        }

        
    }

    //---------------------------------------------------------------------------------------------------

    public function itemLiked()
    {
        return $this->sendResponse(CompactItemResource::collection(Auth::user()->itemLikes));
    }

    //---------------------------------------------------------------------------------------------------

    public function favorites()
    {
        return $this->sendResponse(CompactItemResource::collection(Auth::user()->favorites));
    }

    //---------------------------------------------------------------------------------------------------

    public function addToFavorites(Request $request)
    {
        // return Item::find($request->id)->favedBy->where('id', Auth::user()->id);
        // if(Item::find($request->id)->favedBy()->where('id', $request->id)->exists())
        //     Item::find($request->id)->favedBy()->where('id', $request->id)->detach(['user_id' => Auth::user()->id]);
        Auth::user()->favorites()->attach([$request->id]);
        return $this->sendResponse();
    }

    //---------------------------------------------------------------------------------------------------

    public function deleteFavorites(Request $request)
    {
        Auth::user()->favorites()->detach([$request->id]);
        return $this->sendResponse();
    }

    //---------------------------------------------------------------------------------------------------

    public function getOTPCode(GetOTPRequest $request)
    {

        return $this->sendResponse(['otp_code' => $this->userServices->createOTPCode($request)], 'کد یکبار مصرف فرستاده شد');

    }

    //---------------------------------------------------------------------------------------------------

    public function validateOTPCode(ValidateOTPRequest $request)
    {

        if(Cache::has('OTP' . $request->phone_number)){

            // $otpCode = Cache::get('OTP' . $request->phone_number);

            if ($this->userServices->validateOTPCode($request)) {
                return $this->sendResponse();
            } else {
                return $this->sendError('کد اعتبار سنجی اشتباه است');
            }
        } else {
            return $this->sendError('کد شما منقضی شده است');
        }
        

    }

    

    //---------------------------------------------------------------------------------------------------

    public function logout()
    {
        Auth::user()->tokens()->delete();
        return $this->sendResponse();
    }

    //---------------------------------------------------------------------------------------------------


    public function info()
    {
        // return $this->model->findOrFail(Auth::user()->id);
        $user = Auth::user();
        $subsubscription = [];
        unset($user['subscription']);
        if(!is_null($user->subscription_end)){
            $subsubscription['has'] = now()->lt($user->subscription_end);
        } else {
            $subsubscription['has'] = false;
        }

        $subsubscription['days']    = now()->diffInDays($user->subscription_end) + 1;
        $subsubscription['start']   = $user->subscription_start;
        $subsubscription['end']     = $user->subscription_end;
        $user['subscription'] = $subsubscription;
        return $this->sendResponse($user);
    }

    public function updateUser(UpdateUser $request)
    {
        // return Auth::user()->fill($request->validated())->save();
        return $this->sendResponse(Auth::user()->fill($request->validated())->save());
    }

    public function changePassword(ChangePassword $request)
    {
        // return Auth::user()->fill($request->validated())->save();
        $data = $request->validated();
        // return $data;
        // return $data;
        $user = Auth::user();
        if(Hash::check($data['pre_password'], $user->password)){
            $user->password = Hash::make($data['password']);
            $user->save();
            return $this->sendResponse();
        } else {
            return $this->sendError('رمز عبور فعلی خود را درست وارد کنید');
        }
    }

    public function resetPassword(Request $request)
    {
        // return Auth::user()->fill($request->validated())->save();
        // return $data;
        // return $data;
        $user = Auth::user();
        $user->password = Hash::make($request->password);
        $user->save();
        return $this->sendResponse();
    }

    // public function plans()
    // {
    //     $data = [];
    //     $data['plans'] = Plan::all();

    //     return $this->sendResponse($data);

    // }

}

?>